#include <ModbusMaster.h>

#define RS485_RX  4
#define RS485_TX  15
#define RS485_DIR 2

#define XY_MD02_ID 1 // XY-MD02 Device ID

// instantiate ModbusMaster object
ModbusMaster node;

void preTransmission() {
  digitalWrite(RS485_DIR, 1);
}

void postTransmission() {
  digitalWrite(RS485_DIR, 0);
}

void setup() {
  pinMode(RS485_DIR, OUTPUT);
  digitalWrite(RS485_DIR, 0);

  Serial.begin(115200);

  // Modbus communication runs at 115200 baud
  Serial2.begin(9600, SERIAL_8N1, RS485_RX, RS485_TX); // Rx, Tx

  // Modbus slave ID 1
  node.begin(XY_MD02_ID, Serial2);
  
  // Callbacks allow us to configure the RS485 transceiver correctly
  node.preTransmission(preTransmission);
  node.postTransmission(postTransmission);
}

bool state = true;

void loop() {
  // Read 2 registers starting at address 1
  uint8_t result = node.readInputRegisters(1, 2);
  if (result == node.ku8MBSuccess) {
    float temp = node.getResponseBuffer(0) / 10.0f;
    float humi = node.getResponseBuffer(1) / 10.0f;
    
    Serial.printf("Temp: %.01f *C\n", temp);
    Serial.printf("Humi: %.01f %RH\n", humi);
  }

  delay(1000);
}
